import router from "../router.mjs";
import cfg from "../../../config.json";
import url from "url";
import { queries } from "./inc/index.mjs";
import sql from "../sql.mjs";
import lib from "../lib.mjs";
import tpl from "../tpl.mjs";

tpl.readdir("views");

router.get("/", async (req, res) => {
  const query = await sql.query("select id, mime from items order by id desc limit 300");
  const data = {
    items: query,
    last: query[query.length - 1].id
  };

  res.reply({ body: tpl.render("views/index", data) });
});

router.get(/^\/([0-9]+)$/, async (req, res) => {
  const query = (await sql.query(queries.item, Array(3).fill(req.url.split[0])))?.shift();
  
  if(!query?.id)
    return res.redirect("/404");
  
  const data = {
    user: {
      name: query.username,
      channel: query.userchannel,
      network: query.usernetwork
    },
    item: {
      id: query.id,
      src: {
        long: query.src,
        short: url.parse(query.src).hostname,
      },
      thumbnail: `${cfg.websrv.paths.thumbnails}/${query.id}.png`,
      dest: `${cfg.websrv.paths.images}/${query.dest}`,
      mime: query.mime,
      size: lib.formatSize(query.size),
      timestamp: new Date(query.stamp * 1000).toISOString()
    },
    next: query.next ? query.next : null,
    prev: query.prev ? query.prev : null,
    title: `${query.id} - f0ck.me`
  };
  res.reply({ body: tpl.render("views/item", data) });
});

router.get(/^\/(contact|help|how)$/, (req, res) => {
  res.reply({ body: tpl.render(`views/${req.url.split[0]}`) });
});

router.get("/random", async (req, res) => {
  res.redirect("/" + (await sql.query("select id from items order by rand() limit 1"))[0].id)
});
